/**
  ******************************************************************************************
  * @file   	control.c
  * @brief  	C-Datei mit Threads fr Steuerung der Abstimmung 
	* @version	V1.0.0
  * @date   	2014/06/29
  * @author		Nico Schmidt
  * 
  *  	description: -	Steuergert fr eine Magnetic-Loop Antenne o. .
	*							   		mit XMC1100 auf XMC 2Go-Board zur Teilnahme am 
	*										Infineon XMC Design Contest 2014.
	*                -	Erstellt mit Infinion DAVE 3 und Keil Vision 5.10
	*								 -	Verffentlicht unter CC BY-SA 2.0 DE
	*
	*******************************************************************************************                
  *	In dieser Datei befindet sich der zentrale Thread control(), von dem aus alle
  * Ein- und Ausgangsaktionen koordiniert werden.
  *******************************************************************************************
*/

#include "control.h"

extern osMailQId SWR_Mail_Queue;
extern osMailQId UART_Tx_Mail_Queue;

void control(void const *arg)
{
	
	uint8_t Richtung=0, Geschwindigkeit=0;
	osEvent evt;
	SWR_Queue_struct *SWR;
	float SWR_Value;

	while(1)
	{	
		evt = osMailGet(SWR_Mail_Queue, 1);									      // auf eingehende Mail prfen
			
		if (evt.status == osEventMail)														// neue Mail angekommen?				
		{
				SWR = evt.value.p;																		// Startpointer laden
			
				if (SWR_Value > SWR_Max)															// SWR grer als max in swr.h?
				{
						
						if (SWR_Value > 2.0)															// Bei SWR >2,0
						{
							Richtung = 1;			
							Geschwindigkeit	= 2;														// Schnell drehen 
						}
						else 																							// ansonsten				
						{
							Richtung = 1;							
							Geschwindigkeit	= 1; 														// Langsam drehen
						}
				}
				
				osMailFree(SWR_Mail_Queue, SWR); 					            // Speicher von Mail wieder freigeben
		}
		
		if (IO004_ReadPin(IO004_Handle1)^IO004_ReadPin(IO004_Handle2))	// Taster gedrckt?
		{
				if (IO004_ReadPin(IO004_Handle1))											// Taster links gedrckt?		
				{
							Richtung = 2;																		// Richtung = links	
							Geschwindigkeit	= 1; 														// Langsam drehen
				}				
				else
				{
							Richtung = 1;																		// Richtung = rechts							
							Geschwindigkeit	= 1; 														// Langsam drehen
				}
					
		}
		
		if (Richtung)																							// Motor drehen
		{
				if (Richtung == 1) IO004_SetPin(IO004_Handle4);				// Richtung ausgeben
				else   IO004_ResetPin(IO004_Handle4);								
				
				if (Geschwindigkeit==2)																// Schnelllauf?
					PWMSP001_SetDutyCycle((PWMSP001_HandleType*)				// Duty-Time = 90%	
										&PWMSP001_Handle0,90);
				else																									// ansonsten
					PWMSP001_SetDutyCycle((PWMSP001_HandleType*)					// Duty-Time = 45%
										&PWMSP001_Handle0,45);
				
				PWMSP001_Start((PWMSP001_HandleType*)									// PWM starten
										&PWMSP001_Handle0);
				
		}
		
		osDelay(200);																							// 200ms warten
				
		PWMSP001_Stop((PWMSP001_HandleType*)											// PWM stoppen
		&PWMSP001_Handle0);		
				
		Richtung=0;	 																							// Richtung lschen
		
		if (SWR_Value <= SWR_Max)																	// SWR Ok?
			    IO004_SetPin(IO004_Handle3);												// "Tune" Ausgang auf 1
		else																											// ansonsten
					IO004_ResetPin(IO004_Handle3);											// "Tune" Ausgang auf 0												
		
								
	}
}
