/**
  ******************************************************************************************
  * @file   	swr.c
  * @brief  	C-Datei mit Threads fr SWR-Berechnung 
	* @version	V1.0.0
  * @date   	2014/06/29
  * @author		Nico Schmidt
  * 
  *  	description: -	Steuergert fr eine Magnetic-Loop Antenne o. .
	*							   		mit XMC1100 auf XMC 2Go-Board zur Teilnahme am 
	*										Infineon XMC Design Contest 2014.
	*                -	Erstellt mit Infinion DAVE 3 und Keil Vision 5.10
	*								 -	Verffentlicht unter CC BY-SA 2.0 DE
	*
	*******************************************************************************************                
  *	In dieser Datei befindet sich der Thread swr_messung(), mit dem die Eingangsspannungen
  *	vom SWR-Messkopf gewandelt und der berechnete SWR-Wert an den Haupthread control()
  *	gesendet wird. 
  *******************************************************************************************
*/

#include "swr.h"

extern osMailQId SWR_Mail_Queue;

void swr_messung(void const *arg)
{
	ADC004_ResultHandleType *Result;														// Struktur fr Messergebnisse
	uint16_t Leist_vor, Leist_zur;															// Variablen gemessende Leistung vor und zurck
	float SWR;																									// Variable fr SWR-Wert					
	
	SWR_Queue_struct *transmit;																	// 		
 	
  while(1)																										// Enlosschleife	
	{
			ADC004_SetPendingChannel((ADC004_HandleType*)						// Kanal vor Vorwrtsleistung whlen
											&ADC004_Handle0, &PIN_vor);
			osDelay(1);																							// 1ms zum umschalten warten
			ADC004_GetResult(&ADC004_Handle0, Result);							// Kanal einlesen
			
			Leist_vor = Result->Result;															// Ergebnis zwischenspeichern

			if (Leist_vor >= Min_Leistung)													// Minimalwert fr Vorwrtsleistung erreicht? Dann ...
			{
				ADC004_SetPendingChannel((ADC004_HandleType*)					// Kanal vor Rckwrtsleistung whlen
									&ADC004_Handle0, &PIN_zur);
				osDelay(1);																						// 1ms zum umschalten warten
				ADC004_GetResult(&ADC004_Handle0, Result);						// Kanal einlesen
				
				Leist_vor = Result->Result;														// Ergebnis zwischenspeichern
				
				SWR = (((float)(Leist_vor+Leist_zur))									// Stehwellenverhltnis berechnen
										/(float)(Leist_vor-Leist_zur));
				
				transmit = osMailAlloc(SWR_Mail_Queue,								// Speicher fr Mail resevieren
										 osWaitForever);

				transmit->SWR_Value = SWR;														// SWR eintragen
				
				osMailPut(SWR_Mail_Queue, transmit);									// Mail absenden
			}
		
		osDelay(100);																							// 100 ms bis zur nchsten Messung warten	
	
	}
}




