/**
  ******************************************************************************************
  * @file   	system_init.c
  * @brief  	C-Datei mit Funktionen zur Systeminitialisierung
  * @version	V1.0.0
  * @date   	2014/06/29
  * @author		Nico Schmidt
  * 
  *  	description: -	Steuergert fr eine Magnetic-Loop Antenne o. .
	*							   		mit XMC1100 auf XMC 2Go-Board zur Teilnahme am 
	*										Infineon XMC Design Contest 2014.
	*                -	Erstellt mit Infinion DAVE 3 und Keil Vision 5.10
	*								 -	Verffentlicht unter CC BY-SA 2.0 DE
	*
	*******************************************************************************************                
  *	In dieser Datei befindet sich die Funktion system_init() in der die einzelnen Threads
  * angelegt und gestartet werden. Weiterhin werden die  Mailboxen fr die einzelnen Threads
  * angelegt.
	*	Zustzlich ist der Thread system_running() vorhanden, mit dem ber die auf dem Board
	* befindliche Led 2 durch Blinken signalisiert wird das dass RTOS CMX noch luft  
  *******************************************************************************************
*/

#include "system_init.h"
// ------------------------ Threads definieren ------------------------------------------------
osThreadDef (system_running, osPriorityNormal, 1, 0);	// Definition von Thread "system_running" 
																											// (Zeichen das das System noch luft)
osThreadDef (uart_transmit, osPriorityNormal, 1, 0);	// Definition von Thread "uart_transmit" 
																											// (Zeichen an PC senden)																											
osThreadDef (swr_messung, osPriorityNormal, 1, 0);		// Definition von Thread "swr_messung" 
																											// (Analoge Werte von Pin 2.8 und 2.9 einlesen)
osThreadDef (control, osPriorityNormal, 1, 0);				// Definition von Thread "control" 
																											// (Steuerung aller I/O-Funktionen)
//------------------------- Mailboxen einrichten------------------------------------------------         
osMailQId UART_Tx_Mail_Queue;													// Definition der Mailqueue fr UART Sendemailbox
osMailQId SWR_Mail_Queue;															// Definition der Mailqueuefr SWR-bermittlung

osMailQDef(UART_Tx_Mail_Queue, 16,Uart_Queue_struct);	// Definition der Parameter fr UART Sendemailbox
osMailQDef(SWR_Mail_Queue, 8,SWR_Queue_struct);				// Definition der Parameter fr SWR-bermittlung
//---------------------------------------------------------------------------------------------

void system_init(void)
{
		DAVE_Init();																			// Initialisation der DAVE Apps
	
		osThreadCreate (osThread (system_running), NULL);	// Anlegen von Thread "system_running" 
																											// (Zeichen das das System noch luft)
		osThreadCreate (osThread (uart_transmit), NULL);	// Anlegen von Thread "uart_transmit" 
																											// (Zeichen zum PC senden)
		osThreadCreate (osThread (swr_messung), NULL);		// Anlegen von Thread "swr_messung" 
																											// (Vor- und Rcklaufender Spannungen Messen und
																											//	SWR berechnen)
		osThreadCreate (osThread (control), NULL);				// Anlegen von Thread "control" 
																											// (Steuerung aller I/O-Funktionen)
		UART_Tx_Mail_Queue = osMailCreate(osMailQ					// Mail Queue fr Sendungen ber serielle
												(UART_Tx_Mail_Queue), NULL);	// Schnittstelle anlegen	
		SWR_Mail_Queue = osMailCreate(osMailQ							// Mail Queue fr Sendung des SWR-Wertes
												(SWR_Mail_Queue), NULL);			// anlegen
	
}

void system_running(void const *arg)
{
		/* Anzeige das das Betriebssystem noch luft
			 Led 2 am XMC 2Go blinkt mit 0,5 Hz Takt */
		
		while(1)																					// Endlosschleife	
		{	
			IO004_TogglePin(IO004_Handle0);									// Pin 1.0 (Led 2) umschalten
			
			osDelay(1000);																	// 1000ms warten
		}
}


