/**
  ******************************************************************************************
  * @file   	uart.c
  * @brief  	C-Datei mit Threads fr den UART
	* @version	V1.0.0
  * @date   	2014/06/29
  * @author		Nico Schmidt
  * 
  *  	description: -	Steuergert fr eine Magnetic-Loop Antenne o. .
	*							   		mit XMC1100 auf XMC 2Go-Board zur Teilnahme am 
	*										Infineon XMC Design Contest 2014.
	*                -	Erstellt mit Infinion DAVE 3 und Keil Vision 5.10
	*								 -	Verffentlicht unter CC BY-SA 2.0 DE
	*
	*******************************************************************************************                
  *	In dieser Datei befindet sich der Thread uart_transmit(), mit dem ber Mailbox ankommende
	* Nachrichten ber die serielle Schnittstelle an eine PC o. . gesendet werden. 
  *******************************************************************************************
*/

#include "uart.h"

extern osMailQId UART_Tx_Mail_Queue;

void uart_transmit(void const *arg)
{
	Uart_Queue_struct *transmit;																// Speicher fr Mail resevieren
	osEvent evt;																								// Struktur fr Eventverarbeitung anlegen				
	
	UART002_LocalStructType TxChn;															// Struktur fr UART anlegen
	UART002_LocalStructureInit(&UART002_Handle0,&TxChn,0x0001); // Struktur fr UART initialisieren
	
  while(1)																										// Enlosschleife	
	{
		evt = osMailGet(UART_Tx_Mail_Queue, osWaitForever);       // auf eingehende Mail prfen
			
		if (evt.status == osEventMail)														// neue Mail angekommen?				
		{
				transmit = evt.value.p;																// Startpointer laden
				TxChn.pBuffer = (uint8_t*)transmit->Message;					// Message laden
				TxChn.DataLen = transmit->Lenght;											// Lnge laden		
				UART002_StartTransmission(&TxChn,1000);								// bertragung starten
				UART002_WaitForCompletion(&TxChn,1000);								// auf Ende der bertragung warten	
				osMailFree(UART_Tx_Mail_Queue, transmit);             // Speicher von Mail wieder freigeben

    }
	}
}
